// run immediately once executed
documentWatcher();

function documentWatcher() {
  if (document.querySelector('span.title')) watchTitle();
  if (document.querySelector('span.statusTitle')) watchProblemNumber();
};

function watchTitle() {
  var title = document.querySelector('span.title').textContent;
  // console.log('found title', title);
  sendInfo({ title: true, value: title });
}

// Here, we want to continually check the text value of the element, because
// the problem number will change as the user switches between problems
// (the existing iframe element is reused).
function watchProblemNumber() {
  chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
    sendResponse({message_received: true});
    if (request.sender === 'sidebar' && request.submission) inputSubmission(request.submission);
  });

  var number = newProblemPage();
  setInterval(function() {
    var new_number = document.querySelector('span.statusTitle').textContent;
    if (new_number !== number) {
      number = new_number;
      newProblemPage(new_number);
    }
  }, 1000);
}

function newProblemPage(number) {
  var number = number || document.querySelector('span.statusTitle').textContent;
  // console.log('found problem number', number);
  parseExpressionsOnClick(number);
  sendInfo({ problem_number: true, value: number });
  return number;
}

function inputSubmission(latex) {
  var time_between_events = 150;

  window.focus(); // focus might be on the sidebar iframe

  setTimeout(function() {
    clearInput(input);
  }, 20);

  function clearInput(callback) {
    var input = document.querySelector("input.focusNode");
    var field = document.querySelector("span.inputField");
    var field_doc = document.querySelector("span.inputField .eqDocument");
    var undo_btn = document.querySelector("button[title=Undo]");
    if (!input || !undo_btn || !field_doc) return;
    if (!field || field.className.indexOf('disabled') !== -1) return; // answer already submitted
    input.focus();
    let max_undo = 300;
    while (!field_doc.classList.contains("empty") && (max_undo--) > 0) {
      triggerMouseEvent(undo_btn, "mousedown");
    }
    setTimeout(callback, time_between_events);
  }

  function input() {
    var input = document.querySelector("input.focusNode")
      , left_btn = document.querySelector("button[title=Left]")
      , right_btn = document.querySelector("button[title=Right]")
      , dot_btn = document.querySelector("button[title='Multiplication dot *']");
    if (!input || !left_btn || !right_btn || !dot_btn) return;
    var btns = [left_btn, right_btn, dot_btn];

    var commands = PearsonParser.latexToInputCommands(latex);

    // buttons only work when input is focused
    // input.focus();

    let idx = 0;

    function applyNextCommand() {
      var command = commands[idx];
      idx += 1;
      if (command.input) inputValue(input, command.input);
      else if (command.key) inputKey(btns, command.key);
      if (idx < commands.length) setTimeout(applyNextCommand, time_between_events);
    }
    setTimeout(applyNextCommand, 1);

    function inputValue(input, value) {
      input.value = value;
      triggerHTMLEvent(input, 'input');
    }

    function inputKey(btns, key) {
      var btn;
      if (key === 'left') btn = btns[0];
      else if (key === 'right') btn = btns[1];
      else if (key === 'dot') btn = btns[2];
      triggerMouseEvent(btn, "mousedown");
    }

    function triggerHTMLEvent(node, eventType) {
        var evt = document.createEvent("HTMLEvents");
        evt.initEvent(eventType, true, true ); // event type,bubbling,cancelable
        return node.dispatchEvent(evt);
    }
  }

  function triggerMouseEvent(node, eventType) {
    var clickEvent = document.createEvent('MouseEvents');
    clickEvent.initEvent(eventType, true, true);
    node.dispatchEvent(clickEvent);
  }
}

function parseExpressionsOnClick(number) {
  // need to name the event listener so we can potentially remove it
  function eqDocumentClickListener() {
    // Don't do anything if the user clicked the actual input field contained in the "eqDocument".
    if (this.classList.contains('inputField') || this.parentNode.classList.contains('inputField')) return;

    var parsed_ascii = pearsonParser.parsePearsonExpression(this);
    sendParsedPearsonExpression(parsed_ascii, number);
  }

  // Normal cases
  addEventListenerList(document.querySelectorAll('span.eqDocument'), 'click', eqDocumentClickListener);

  // Cases where the author of the rendered expression provides redundant
  // information about terms and adds unnecessary span wrappers.  Must remove
  // event listeners from the eqDocument spans within this expression, otherwise
  // the expression won't be parsed as a whole.
  var nowrap_divs = document.querySelectorAll('div.nowrap');
  for (var i=0; i<nowrap_divs.length; i++) {
    removeEventListenerList(nowrap_divs[i].querySelectorAll('span.eqDocument'), 'click', eqDocumentClickListener);

    nowrap_divs[i].addEventListener('click', function() {
      // Don't do anything if the user clicked the actual input field contained in the "nowrap" div.
      if (this.classList.contains('inputField') || this.parentNode.classList.contains('inputField')) return;

      // We allow expressions that contain input fields to be parsed on click.
      // But, we do not want to insert an expression on the canvas if the input
      // field is clicked.  If the input field is in focus, then we know this
      // is the case.
      if (this.querySelector('input:focus')) return;

      var parsed_ascii = pearsonParser.parsePearsonExpression(this);
      sendParsedPearsonExpression(parsed_ascii, number);
    });
  }
}

function sendParsedPearsonExpression(ascii, problem_number) {
  chrome.runtime.sendMessage({ sender: 'document watcher'
                             , expression: ascii
                             , problem_number: problem_number }, function(response) {
    // console.log(response);
  })
}

function sendInfo(info) {
  chrome.runtime.sendMessage(Object.assign({ sender: 'document watcher' }, info), function(response) {
    // console.log('have we found everything?', response.analysis_done);
  });
}

function addEventListenerList(list, event, fn) {
  for (var i = 0; i < list.length; i++) {
    list[i].addEventListener(event, fn, false);
  }
}

function removeEventListenerList(list, event, fn) {
  for (var i = 0; i < list.length; i++) {
    list[i].removeEventListener(event, fn, false);
  }
}
